@@ -3,6 +3,8 @@
 //! A library for basic image filtering.
 
 pub mod img;
+pub mod filters;
+pub mod misc;
 pub(self) mod utils;
 
 pub use img::Img;
